// Created by Sebastien Salmon
// M3M - UTBM
// sebastien[.]salmon[@]utbm[.]fr
// 2010
// released under CC-BY-NC-SA

function PSO_inertial_radius(objective,wmax,wmin,itmax,c1,c2,N,D,borne_sup,borne_inf,vitesse_min,vitesse_max,radius)

lines(0)

//---------------------------------------------------
// Setting up graphics axis
//---------------------------------------------------
      scf(1)
      gcf()
      xtitle("Objective function value vs Iteration")
      axe_prop=gca()
      axe_prop.x_label.text="Iteration number"
      axe_prop.y_label.text="Objective function value"
      
      scf(2)
      gcf()
      xtitle("Swarm radius vs Iteration")
      axe_prop=gca()
      axe_prop.x_label.text="Iteration number"
      axe_prop.y_label.text="Swarm radius (log10)"

      
      

//---------------------------------------------------
// Declaring objective function
//---------------------------------------------------
var=objective+'.sce';
exec(var) 

//---------------------------------------------------
// PSO parameters definition
//---------------------------------------------------

// swarm limit radius
// radius=1e-4; // limit radius of the swarm, if inf then stopping process

// using inertial weigth parameter improvement
//wmax=0.9; // initial weigth parameter
//wmin=0.4; // final weigth parameter

// max iteration allowed
//itmax=800; //Maximum iteration number

// knowledge factors
//c1=2; // for personnal best
//c2=2; // for global best

// problem dimensions
//N=20; // number of particles
//D=6; // problem dimension

//---------------------------------------------------
// Allocation of memory ans first computations
//---------------------------------------------------

// computation of the weigth vector
for i=1:itmax
W(i)=wmax-((wmax-wmin)/itmax)*i;
end

// computation of location and speed of particles

for i=1:D
  
//borne_sup(i)= 1;
//borne_inf(i)= 0;
x(1:N,i) =borne_inf(i) +rand(N,1) * ( borne_sup(i) - borne_inf(i) ); // location

//vitesse_min(i)=-0.3;
//vitesse_max(i)=0.3;
v(1:N,i)=vitesse_min(i)+(vitesse_max(i)-vitesse_min(i))*rand(N,1); // speed
end

// actual iteration number
j=1;

// actual mesurability status
mesurability=1;
counter=0;

//---------------------------------------------------
// First evaluation of the objective function
//---------------------------------------------------
for i=1:N  
    y=x(i,:,j);
    F(i,1,j)=script(y); // mono-objective result
end


//---------------------------------------------------
// Search for the minimum of the swarm
//---------------------------------------------------
[C,I]=min((F(:,1,j)));

//---------------------------------------------------
// The first minimun is the global minimum 'cause first
// iteration
//---------------------------------------------------
gbest(1,:,j)=x(I,:,j);

for p=1:N
    G(p,:,j)=gbest(1,:,j); // creating a matrix of gbest, used for speed computation
end

//---------------------------------------------------
// The first minimun is the best result 'cause first
// iteration
//---------------------------------------------------
Fbest(1,1,j)=F(I,1,j); // global best
Fb(1,1,j)=F(I,1,j); // iteration best, used for comparison with global best

//---------------------------------------------------
// Each particle is her personnal best 'cause first
// first iteration
//---------------------------------------------------    
for i=1:N
  pbest(i,:,j)=x(i,:,j);
end
    
//---------------------------------------------------
// Speed and location computation for next iteration
//---------------------------------------------------    
v(:,:,j+1)=W(j)*v(:,:,j)+c1*rand()*(pbest(:,:,j)-x(:,:,j))+c2*rand()*(G(:,:,j)-x(:,:,j)); // speed
x(:,:,j+1)=x(:,:,j)+v(:,:,j+1); // location

//---------------------------------------------------
// Entering to the optimization loop
//---------------------------------------------------
while (j<itmax-1 & mesurability==1)
  j=j+1
  
  
    //---------------------------------------------------
    // First evaluation of the objective function
    //---------------------------------------------------
    for i=1:N
      y=x(i,:,j);
      F(i,1,j)=script(y);    
    end
    
    //---------------------------------------------------
    // Search for the minimum of the swarm
    //---------------------------------------------------
    [C,I]=min((F(:,:,j)));

    //---------------------------------------------------
    // Searching for global minimum
    //---------------------------------------------------
    gbest(1,:,j)=x(I,:,j); // hypothesis : this iteration is better than last one
    Fb(1,1,j)=F(I,1,j); // looking for the iteration best result
    Fbest(1,:,j)=Fb(1,:,j); // hypothesis : Fbest is the iteration best result
    [C,I]=min(Fb(1,1,:)); // looking if it's the global best
       
    if Fb(1,1,j)<C
       
      else
        gbest(1,:,j)=gbest(1,:,I(3)); // if not then replacing with the good gbest
        Fbest(1,:,j)=Fbest(1,:,I(3)); // A new Fbest has not be found this time
    end 
   
    for p=1:N
      G(p,:,j)=gbest(1,:,j); // creating a matrix of gbest, used for speed computation
    end

    //---------------------------------------------------
    // Computation of the new personnal best
    //---------------------------------------------------    
    for i=1:N
      [C,I]=min(F(i,1,:));
        if F(i,1,j)<C
           pbest(i,:,j)=x(i,:,j);
         else
        pbest(i,:,j)=x(i,:,I(3));
        end
     end


    //---------------------------------------------------
    // Re-computation of Fbest for reliability test
    //---------------------------------------------------    
    y=gbest(1,:,j);
    Fbest(:,:,j)=script(y);
     
     
     //---------------------------------------------------
     // Speed and location computation for next iteration
     //--------------------------------------------------- 
      v(:,:,j+1)=W(j)*v(:,:,j)+c1*rand()*(pbest(:,:,j)-x(:,:,j))+c2*rand()*(G(:,:,j)-x(:,:,j)); // speed
      x(:,:,j+1)=x(:,:,j)+v(:,:,j+1); // location    
      
     //---------------------------------------------------
     // Computing measurability - generating capacity
     //--------------------------------------------------- 
      [C,I]=min((F(:,:,j)));
      leader_temp(1,:,j)=x(I,:,j); // getting swarm leader
   
      for t=1:N
         dist_temp(:,t,j) = abs((x(t,:,j)-leader_temp(:,:,j))); // computing L1 norm to leader
      end

      for t=1:N
          dist(t,j)=max(dist_temp(:,t,j));
      end
      
      max_dist(j)=max(dist(:,j));
      
      if max_dist(j)>=radius & counter<10
        mesurability=1;
        counter=0;
      else
       
        counter=counter+1;
                       
       end
       
       if counter==10
           mesurability=0;
       end
       
       
       rad_plot(j)=radius;
       
       if modulo(j,25)==0 // resetting graphics
           clf(2)
           scf(2)
           gcf()
           xtitle("Swarm radius vs Iteration")
           axe_prop=gca()
           axe_prop.x_label.text="Iteration number"
           axe_prop.y_label.text="Swarm radius (log10)"
       else
           scf(2)
       end
       plot(log10(max_dist(2:j)) )
       plot(log10(rad_plot(2:j)),'r')
       drawnow()
      
     //---------------------------------------------------
     // Plotting the Fbest curve to monitor optimization
     //--------------------------------------------------- 
      for count=1:j
        Fbest_draw(count)=Fbest(1,1,count);
      end
  
      if modulo(j,25)==0 // resetting graphics

          clf(1)
          scf(1)
          gcf()
          xtitle("Objective function value vs Iteration")
          axe_prop=gca()
          axe_prop.x_label.text="Iteration number"
          axe_prop.y_label.text="Objective function value"



      else

          scf(1)


      end
      plot(Fbest_draw)
      drawnow()
      
      
     //---------------------------------------------------
     // Temporary save in case of crash, very usefull for long time optimization
     //--------------------------------------------------- 
      save ('PSO_temp')

//---------------------------------------------------
// Out of the optimization loop
//---------------------------------------------------
end       

disp('Fbest :' + string(Fbest(:,:,j)))
disp('Gbest :' + string(gbest(:,:,j)))

save('results_PSO_radius')

endfunction
